<?php
/**
 * the legal disclaimer, etc.
 *
 * This script reflects meta-information listed in page headers.
 *
 * It also displays the mail address of the webmaster, if any.
 *
 * @link http://www.bebits.com/about
 * @link http://www.bebits.com/legal
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @tester Spilo
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// common definitions and initial processing
include_once 'shared/global.php';

// load localized strings
i18n::bind('root');

// load the skin
load_skin('about');

// the title of the page
$context['page_title'] = i18n::s('About this server');

// what are we talking about?
$context['text'] .= '[title]'.sprintf(i18n::s('What is %s about?'), $context['site_name'])."[/title]\n";

$context['text'] .= '<p>'.i18n::s('Welcome to this server! Here are attributes transmitted in the header of each page:')."</p>\n".Skin::table_prefix('100%');

// site name
if(trim($context['site_description']))
	$context['text'] .= Skin::table_row(array(i18n::s('Site name'), $context['site_name']));

// site description
if(trim($context['site_description']))
	$context['text'] .= Skin::table_row(array(i18n::s('Description'), $context['site_description']));

// site keywords
if(trim($context['site_keywords']))
	$context['text'] .= Skin::table_row(array(i18n::s('Keywords'), $context['site_keywords']));

// site position
if(trim($context['site_position']))
	$context['text'] .= Skin::table_row(array(i18n::s('Geographical position'), $context['site_position']));

// site copyright
if(trim($context['site_copyright']))
	$context['text'] .= Skin::table_row(array(i18n::s('Copyright'), $context['site_copyright']));

// site owner
if(trim($context['site_owner']))
	$context['text'] .= Skin::table_row(array(i18n::s('Owner'), $context['site_owner']));
else
	$context['site_owner'] = '??';

$context['text'] .= Skin::table_suffix()."\n";

// powered by
$context['text'] .= '<p>'.sprintf(i18n::s('This site is powered by %s, %s, and %s - a fast, easy-to-use site that lets you access, review, and download information that matters for you.'),
	Skin::build_link(i18n::s('http://www.yetanothercommunitysystem.com/'), i18n::s('YACS')),
	Skin::build_link(i18n::s('http://www.php.net/'), i18n::s('PHP')),
	Skin::build_link(i18n::s('http://www.mysql.com/'), i18n::s('MySQL')))."</p>\n";

// contact us
$context['text'] .= '[title]'.i18n::s('Contact Us')."[/title]\n";

// query form
$context['text'] .= i18n::s('<p>The preferred mean to join us is to use the general-purpose query form. This is a convenient tool for you and for us, because your request and related answers will be listed at a single place accessible from your web browser. Moreover, you will be informed by e-mail of any progressing step to your query.</p>')."\n";
$context['text'] .= '<p class="indent">'.sprintf(i18n::s('Query: %s'), Skin::build_link('query.php', i18n::s('Use the on-line form'), 'shortcut'))."</p>\n";

// mail address
$context['text'] .= i18n::s('<p>It seems to be one of the Laws of the Internet that every site of any importance must have an email address for the webmaster. Far be it from us to break the trend; so, if you have something vitally important to tell the folks who designed this place, send a message at the following address.</p>')."\n"
	.'<p class="indent">'.sprintf(i18n::s('Webmaster: %s'), $context['site_email'])."</p>\n"
	.Skin::build_block(i18n::s('Due to the large amount of e-mail that we get, we can NOT guarantee a response! This is especially true if your e-mail concerns issues we can\'t really help you with.'), 'note')."\n";

// how to help us?
$context['text'] .= '[title]'.i18n::s('How you can help')."[/title]\n";

$context['text'] .= i18n::s('<p>We\'re proud of this server, but it certainly isn\'t perfect. We\'re always looking for ways to improve the site. So, if you\'ve got any great ideas, then please let us know about them. Or, if you know of some important information that isn\'t listed here, email the author and tell them you want it listed on this site! It\'ll help make this server a better site for the whole community - and that\'s who we\'re here to serve.</p>')."\n";

// legal stuff section
$context['text'] .= '[title]'.i18n::s('Legal Stuff')."[/title]\n";

// be nice
$context['text'] .= i18n::s('<p>Be nice to other people, don\'t spam them, and don\'t do anything bad with any information you might get from this site.</p>')."\n";

// owner
$context['text'] .= sprintf(i18n::s('<p>All content, including but not limited to user reviews, application listings and information contained therein, is owned by %s and may not be copied without permission. However, the above content my be freely modified and/or removed by the person or persons responsible for submitting it.</p>'), $context['site_owner'])."\n";

// liability
$context['text'] .= sprintf(i18n::s('<p>%1$s is not responsible for the content of information submitted to the site. %1$s is not responsible for any content contained on other sites that may link to or be linked to %2$s. As such, the existence of a link to another site on %2$s does not express endorsement by %1$s of said site or its contents.</p>'), $context['site_owner'], $context['site_name'])."\n";

// trademarks
$context['text'] .= sprintf(i18n::s('<p>All other trademarks, icons, and logos, shown or mentioned, are the property of their respective owners, including those associated with any solutions listed in %2$s. Although %1$s does not own and is not responsible for all of the content on the site, we reserve the right to edit or remove any content at any time in any way we deem necessary, without any notification whatselver to the owner(s) of that content, to keep it better in line with our stated and/or unstated policies. %1$s is not responsible for any copyright laws violated by the applications listed herein, although we will do everything we can do resolve any disputes that may arise in this area.</p>'), $context['site_owner'], $context['site_name'])."\n";

// insert a table of content
$context['text'] = Codes::beautify("[toc]".$context['text']);

// the date of last modification
if(Surfer::is_associate())
	$context['text'] = '<p>'.i18n::s('Edited').' '.Skin::build_date(getlastmod()).'</p>'.$context['text'];

// the help panel
if(Surfer::is_associate())
	$context['extra'] .= Skin::build_box(i18n::s('Help'), sprintf(i18n::s('<p>To change information displayed, go to the %s.</p>'), Skin::build_link('skins/configure.php', i18n::s('configuration panel for skins'), 'shortcut')), 'navigation', 'help');

// render the skin
render_skin();

?>