<?php
/**
 * layout articles in a digest
 *
 * @see articles/articles.php
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */
Class Layout_articles_as_digest extends Layout_interface {

	/**
	 * list articles
	 *
	 * @param resource the SQL result
	 * @return string the rendered text
	 *
	 * @see skins/layout.php
	**/
	function &layout(&$result) {
		global $context;

		// empty list
		if(!SQL::count($result)) {
			$output = array();
			return $output;
		}

		// load localized strings
		i18n::bind('articles');

		// we return an array of ($url => $attributes)
		$items = array();

		// process all items in the list
		while($item =& SQL::fetch($result)) {

			// reset the rendering engine between items
			if(is_callable(array('Codes', 'initialize')))
				Codes::initialize(Articles::get_url($item['id'], 'view', $item['nick_name']));

			// url to read the full article
			$url = $context['url_to_home'].$context['url_to_root'].Articles::get_url($item['id'], 'view', $item['nick_name']);

			// time of publication
			if(isset($item['publish_date']) && ($item['publish_date'] > NULL_DATE))
				$time = $item['publish_date'];
			else
				$time = NULL_DATE;

			// the title as the label
			$label = ucfirst(trim(strip_tags(Codes::beautify($item['title']))));

			// the section
			$section = '';
			if($item['anchor'] && ($anchor = get_anchor($item['anchor'])))
				$section = ucfirst(trim(strip_tags(Codes::beautify($anchor->get_title()))));

			// the icon to use
			$icon = '';
			if($item['thumbnail_url'])
				$icon = $item['thumbnail_url'];
			elseif($item['anchor'] && ($anchor = get_anchor($item['anchor'])))
				$icon = $anchor->get_thumbnail_url();
			if($icon)
				$icon = $context['url_to_home'].$context['url_to_home'].$icon;

			// the author(s) is an e-mail address, according to rss 2.0 spec
			$author = '';
			if(isset($context['with_author_information']) && ($context['with_author_information'] == 'Y')) {
				if(isset($item['create_address']))
					$author .= $item['create_address'];
				if(isset($item['create_name']) && trim($item['create_name']))
					$author .= ' ('.$item['create_name'].')';
			}

			// some introductory text for this article
			include_once $context['path_to_root'].'articles/article.php';
			$article =& new Article();
			$article->load_by_content($item);
			$introduction = $article->get_teaser('basic');

			// warns on restricted access
			if(isset($item['active']) && ($item['active'] != 'Y'))
				$introduction = '['.i18n::c('Restricted to members').'] '.$introduction;

			// fix references
			$introduction = preg_replace('/"\//', '"'.$context['url_to_home'].'/', $introduction);

			// list all components for this item
			$items[$url] = array($time, $label, $author, $section, $icon, $introduction);

		}

		// end of processing
		SQL::free($result);
		return $items;
	}

}

?>