<?php
/**
 * populate articles
 *
 * Creates articles with following nick names:
 * - 'cover' - A sample cover article for your front page
 * - 'extra_rss' - A sample extra box to link to our XML RSS feed
 * - 'menu' - A sample general menu displayed on all pages
 *
 * Also, if the parameter $context['populate'] is set to 'samples', additional articles will be created:
 * - 'extra' - A sample extra box at the front page
 * - 'gadget_cloud' - A sample gadget box featuring the cloud at the front page
 * - 'gadget_collections' - A sample gadget box featuring available collections at the front page
 * - 'my_article' - A sample plain article
 * - 'my_blog_page' - Some blogging sample
 * - 'my_manual_page' - A sample page of an electronic book
 * - 'my_jive_thread' - A sample thread made of one article, with a number of comments
 * - 'my_thread' - A sample thread made of one article, with a number of comments
 * - 'my_wiki_page' - Some wiki sample content
 * - 'my_yabb_thread' - A sample thread made of one article, with a number of comments
 * - 'navigation' - A sample navigation box
 *
 * @see control/populate.php
 *
 * This script do not use the standard publication mechanism ([code]Articles::publish()[/code])
 * and do not put created pages into weekly and monthly categories.
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// if it was a HEAD request, stop here
if(isset($_SERVER['REQUEST_METHOD']) && ($_SERVER['REQUEST_METHOD'] == 'HEAD'))
	return;

// this script will be included most of the time
$included = TRUE;

// but direct call is also allowed
if(count(get_included_files()) < 3) {
	$included = FALSE;

	// include global declarations
	include_once '../shared/global.php';

	// load localized strings
	i18n::bind('articles');

	// load the skin
	load_skin('articles');

	// the path to this page
	$context['path_bar'] = array( 'articles/' => i18n::s('Articles') );

	// the title of the page
	$context['page_title'] = i18n::s('Create default pages');

	// stop hackers the hard way
	if(!Surfer::is_associate())
		exit('You are not allowed to perform this operation.');

// load localized strings
} else
	i18n::bind('articles');

// clear the cache for articles
Cache::clear('articles');

// this page is dedicated to articles
include_once $context['path_to_root'].'articles/articles.php';
include_once $context['path_to_root'].'sections/sections.php';
$text = '';

// 'cover' article - basic data
if(Articles::get('cover'))
	$text .= i18n::s('A cover article already exists.').BR."\n";
elseif($anchor = Sections::lookup('covers')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'cover';
	$fields['title'] = i18n::c('Welcome!');
	$fields['introduction'] = sprintf(i18n::c('Cover article %s'), date("m/d/y"));
	$fields['description'] = i18n::s("This is the most recent article posted in the [link=cover section]sections/view.php?id=covers[/link].\nVisit the section to create a new cover article, or to [link=modify this one]articles/edit.php?id=cover[/link].\nIf you don't know how to proceed, visit the [link=help index]help.php[/link].");
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'extra' article - sample data
if(!isset($context['populate']) || ($context['populate'] != 'samples'))
	;
elseif(Articles::get('extra'))
	$text .= i18n::s('A sample extra box already exists.').BR."\n";
elseif($anchor = Sections::lookup('extra_boxes')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'extra';
	$fields['title'] = i18n::c('Hello again');
	$fields['introduction'] = '';
	$fields['description'] = i18n::c("This is a sample extra box.\nVisit the [link=extra section]sections/view.php?id=extra_boxes[/link] to review all extra boxes.");;
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'extra_rss' article - basic data
if(Articles::get('extra_rss'))
	$text .= i18n::s('A RSS feeding extra box already exists.').BR."\n";
elseif($anchor = Sections::lookup('extra_boxes')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'extra_rss';
	$fields['title'] = i18n::c('News Feeder');
	$fields['introduction'] = '';
	$fields['description'] = sprintf(i18n::s('You can get %s from this site. Other %s are also available.'), Skin::build_link('feeds/rss_2.0.php', 'RSS news', 'xml'), Skin::build_link('feeds/', 'information channels', 'shortcut'));
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'gadget_cloud' article - sample data
if(!isset($context['populate']) || ($context['populate'] != 'samples'))
	;
elseif(Articles::get('gadget_cloud'))
	$text .= i18n::s('A sample cloud box already exists.').BR."\n";
elseif($anchor = Sections::lookup('gadget_boxes')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'gadget_cloud';
	$fields['title'] = i18n::c('Tags cloud');
	$fields['introduction'] = '';
	$fields['description'] = '[cloud]';
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'gadget_collections' article - sample data
if(!isset($context['populate']) || ($context['populate'] != 'samples'))
	;
elseif(Articles::get('gadget_collections'))
	$text .= i18n::s('A sample collections box already exists.').BR."\n";
elseif($anchor = Sections::lookup('gadget_boxes')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'gadget_collections';
	$fields['title'] = i18n::c('Collections');
	$fields['introduction'] = '';
	$fields['description'] = '[collections]';
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'menu' article - basic data
if(Articles::get('menu'))
	$text .= i18n::s('A navigation menu already exists.').BR."\n";
elseif($anchor = Sections::lookup('menus')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'menu';
	$fields['title'] = i18n::c('Menu');
	$fields['introduction'] = '';
	$fields['description'] = i18n::c("[search]\n[menu=Home]/[/menu]\n[submenu=Site Map]sections/[/submenu]\n[submenu=Categories]categories/[/submenu]\n[submenu=Users]users/[/submenu]\n[submenu=Help]help.php[/submenu]\n\n[anonymous][---][login][/anonymous]");
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'my_article' article - sample data
if(!isset($context['populate']) || ($context['populate'] != 'samples'))
	;
elseif(Articles::get('my_article'))
	$text .= i18n::s('A sample article already exists.').BR."\n";
elseif($anchor = Sections::lookup('my_section')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'my_article';
	$fields['title'] = i18n::c('Edit me');
	$fields['introduction'] = i18n::c('Sample article');
	$fields['description'] = i18n::c('This is a sample article to let you learn and practice.')
		."\n\nabcdefghijklmnopqrstuvwxyz"
		." &&eacute;\"'(-&egrave;_&ccedil;&agrave; )=~#{[|`\\x^@]}"
		." ^&ecirc;&icirc;&ocirc;&ugrave;%*,;:!?./"
		."\a\b\c\d\e\f\g\h\i\j\k\l\m\o\p\q\r\s\t\u\v\w\x\y\z"
		."\n:) 8) :D :O ;) :( X( !! ?? ?! -- ++ >> "
		."\n[b]bold [i]and[/b] italic[/i][nl][u]underlined[/u]"
		."\n<a href=\"http://myweb/mypage.html\">anchor</a><!-- comment -->"
		."\nCheck my [link=document]http://myweb/mypage.html[/link] on this subject;"
		." more info [email=here>>]me@dummy.com[/email]"
		."\n[code]// say hello\necho \"hello world\";[/code]"
		."\n[quote]Once upon a time...[/quote]"
		."\n[*]First item[nl][nl][*]Second item"
		."\n[list][*]First item [*]Second item[/list]"
		."\nLorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum. "
		."\nLorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum. ";;
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'my_blog_page' article - sample data
if(!isset($context['populate']) || ($context['populate'] != 'samples'))
	;
elseif(Articles::get('my_blog_page'))
	$text .= i18n::s('A sample page exists for the blog.').BR."\n";
elseif($anchor = Sections::lookup('my_blog')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'my_blog_page';
	$fields['title'] = i18n::c('Sample page of the blog');
	$fields['introduction'] = i18n::c('Sample content with its set of notes');
	$fields['description'] = i18n::c('This page demonstrates the rendering of the ##daily## layout.');
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'my_jive_thread' article - sample data
if(!isset($context['populate']) || ($context['populate'] != 'samples'))
	;
elseif(Articles::get('my_jive_thread'))
	$text .= i18n::s('A sample jive thread already exists.').BR."\n";
elseif($anchor = Sections::lookup('my_jive_board')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'my_jive_thread';
	$fields['title'] = i18n::c('Sample jive thread');
	$fields['introduction'] = i18n::c('Sample article with its set of comments');
	$fields['description'] = i18n::c('This page demonstrates the rendering of the ##jive## layout.');
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'my_manual_page' article - sample data
if(!isset($context['populate']) || ($context['populate'] != 'samples'))
	;
elseif(Articles::get('my_manual_page'))
	$text .= i18n::s('A sample page exists for the manual.').BR."\n";
elseif($anchor = Sections::lookup('my_manual_chapter')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'my_manual_page';
	$fields['title'] = i18n::c('Sample page of the manual');
	$fields['introduction'] = i18n::c('Sample content with its set of notes');
	$fields['description'] = i18n::c('This page demonstrates the rendering of the ##manual## layout.');
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'my_thread' article - sample data
if(!isset($context['populate']) || ($context['populate'] != 'samples'))
	;
elseif(Articles::get('my_thread'))
	$text .= i18n::s('A sample thread already exists.').BR."\n";
elseif($anchor = Sections::lookup('my_section')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'my_thread';
	$fields['title'] = i18n::c('Sample thread');
	$fields['introduction'] = i18n::c('Sample article with its set of comments');
	$fields['description'] = i18n::c('Change the layout of this article, or section, to look at various rendering options supported by YACS.');
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'my_wiki_page' article - sample data
if(!isset($context['populate']) || ($context['populate'] != 'samples'))
	;
elseif(Articles::get('my_wiki_page'))
	$text .= i18n::s('A sample page exists for the wiki.').BR."\n";
elseif($anchor = Sections::lookup('my_wiki')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'my_wiki_page';
	$fields['title'] = i18n::c('Sample page of the wiki');
	$fields['introduction'] = i18n::c('Sample content with its set of extensions');
	$fields['description'] = i18n::c('This page demonstrates the rendering of the ##wiki## layout.');
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'my_yabb_thread' article - sample data
if(!isset($context['populate']) || ($context['populate'] != 'samples'))
	;
elseif(Articles::get('my_yabb_thread'))
	$text .= i18n::s('A sample yabb thread already exists.').BR."\n";
elseif($anchor = Sections::lookup('my_yabb_board')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'my_yabb_thread';
	$fields['title'] = i18n::c('Sample yabb thread');
	$fields['introduction'] = i18n::c('Sample article with its set of comments');
	$fields['description'] = i18n::c('This page demonstrates the rendering of the ##yabb## layout.');
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// 'navigation' article - sample data
if(!isset($context['populate']) || ($context['populate'] != 'samples'))
	;
elseif(Articles::get('navigation'))
	$text .= i18n::s('A sample navigation box already exists.').BR."\n";
elseif($anchor = Sections::lookup('navigation_boxes')) {
	$fields = array();
	$fields['anchor'] = $anchor;
	$fields['nick_name'] = 'navigation';
	$fields['title'] = i18n::c('Hello World');
	$fields['introduction'] = '';
	$fields['description'] = i18n::c("This is a sample navigation box.\nVisit the [link=navigation section]sections/view.php?id=navigation_boxes[/link] to review all navigation boxes.");
	$fields['active'] = 'Y';
	$fields['publish_name'] = Surfer::get_name();
	$fields['publish_id'] = Surfer::get_id();
	$fields['publish_address'] = Surfer::get_email_address();
	$fields['publish_date'] = gmstrftime('%Y-%m-%d %H:%M:%S');
	if(Articles::post($fields))
		$text .= sprintf(i18n::s('An article %s has been created.'), $fields['title']).BR."\n";
	else
		$text .= Skin::error_pop().BR."\n";
}

// report on actions performed
if($included)
	echo $text;
else {
	$context['text'] .= $text;
	$menu = array('articles/' => i18n::s('Back to the index page'));
	$context['text'] .= Skin::build_list($menu, 'menu_bar')."\n";
	render_skin();
}
?>