<?php
/**
 * the index page for categories
 *
 * @todo categorize images (NickR)
 * @todo categorize files (NickR)
 * @todo categorize users
 *
 * Any article can be associated to a variable number of categories.
 *
 * For a more comprehensive description of categories, you should check the database abstraction script
 * at [script]categories/categories.php[/script].
 *
 * This page list regular categories to any surfer.
 * Here, regular means that these categories are not hidden and that they have not hit some deadline.
 *
 * For associates, this index has a second part to list hidden or dead categories.
 * Hidden categories are used to locate special items such as featured pages.
 *
 * Following restrictions apply:
 * - anonymous users can see only active categories (the 'active' field == 'Y')
 * - members can see active and restricted categories ('active field == 'Y' or 'R')
 * - associates can see all categories
 *
 * The main menu for this page has a command to create a new category, if necessary.
 * It also has navigation links to browse categories by page, for sites that have numerous categories.
 *
 * Categories are displayed using the default 2-column layout.
 *
 * Because this script is the top of the categories tree, it has a special extra box with helpful links.
 * If the database has more categories than what can be displayed on a single page,
 * another extra box will be displayed to list popular categories.
 *
 * Accept following invocations:
 * - index.php (view the 20 top categories)
 * - index.php/2 (view categories 41 to 60)
 * - index.php?page=2 (view categories 41 to 60)
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @tester Eoin
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// common definitions and initial processing
include_once '../shared/global.php';

// which page should be displayed
$page = 1;
if(isset($_REQUEST['page']))
	$page = $_REQUEST['page'];
elseif(isset($context['arguments'][0]))
	$page = $context['arguments'][0];
$page = strip_tags($page);

// root cache id for this page
$root_cache_id = 'categories/index.php/'.$page;

// render a previous version, if any
Cache::pass_through($root_cache_id);

// load localized strings
i18n::bind('categories');

// load the skin
load_skin('categories');

// the title of the page
$context['page_title'] = i18n::s('The categories tree');

// count categories in the database
include_once 'categories.php';
$stats = Categories::stat_for_anchor(NULL);
if($stats['count'] > CATEGORIES_PER_PAGE) {

	// display the total count of categories
	$context['page_menu'] = array_merge($context['page_menu'], array('_count' => sprintf(i18n::s('%d&nbsp;categories'), $stats['count'])));

	// navigation commands for categories, if necessary
	$home = 'categories/index.php';
	if(isset($context['with_friendly_urls']) && ($context['with_friendly_urls'] == 'Y'))
		$prefix = $home.'/';
	elseif(isset($context['with_friendly_urls']) && ($context['with_friendly_urls'] == 'R'))
		$prefix = $home.'/';
	else
		$prefix = $home.'?page=';
	$context['page_menu'] = array_merge($context['page_menu'], Skin::navigate($home, $prefix, $stats['count'], CATEGORIES_PER_PAGE, $page));
}

// link to site cloud
if($stats['count'])
    $context['page_menu'] = array_merge($context['page_menu'], array( 'categories/cloud.php' => i18n::s('Cloud of tags') ));

// commands for associates
if(Surfer::is_associate())
    $context['page_menu'] = array_merge($context['page_menu'], array( 'categories/edit.php' => i18n::s('Create a category'),
		'categories/check.php' => i18n::s('Maintenance') ));

// the list of active categories
$cache_id = 'categories/index.php#active_categories#'.$page;
if(!$text =& Cache::get($cache_id)) {

	// do it the Yahoo! style
	include_once '../categories/layout_categories_as_yahoo.php';
	$layout =& new Layout_categories_as_yahoo();

	// query the database and layout that stuff
	$offset = ($page - 1) * CATEGORIES_PER_PAGE;
	if(!$text = Categories::list_by_title_for_anchor(NULL, $offset, CATEGORIES_PER_PAGE, $layout))
		$text = '<p>'.i18n::s('No category has been created yet.').'</p>';

	// we have an array to format
	elseif(@count($text))
		$text =& Skin::build_list($text, '2-columns');

	// make a box
	if($text)
		$text =& Skin::build_box('', $text, 'section', 'categories');

	// cache this to speed subsequent queries
	Cache::put($cache_id, $text, 'categories');
}
$context['text'] .= $text;

// associates may list specific categories as well
if(($page == 1) && Surfer::is_associate()) {
	$cache_id = 'categories/index.php#inactive_categories';
	if(!$text =& Cache::get($cache_id)) {

		// query the database and layout that stuff
		if($text = Categories::list_inactive_by_title(0,25)) {

			// we have an array to format
			if(@count($text))
				$text = Skin::build_list($text, '2-columns');

			// displayed as another page section
			$text = Skin::build_block(i18n::s('Special categories'), 'title').$text;
		}

		// cache this to speed subsequent queries
		Cache::put($cache_id, $text, 'categories');
	}
	$context['text'] .= $text;
}

// display extra information
$cache_id = 'categories/index.php#extra';
if(!$text =& Cache::get($cache_id)) {

	// add an extra box with helpful links
	$links = array('sections/' => i18n::s('Site map'),
		'search.php' => i18n::s('Search on keyword'),
		'help.php' => i18n::s('Help index'),
		'query.php' => i18n::s('Contact the webmaster'));
	$text .= Skin::build_box(i18n::s('See also'), Skin::build_list($links, 'compact'), 'extra')."\n";

	// side bar with the list of most popular articles, if this server is well populated
	if($stats['count'] > CATEGORIES_PER_PAGE) {
		if($items = Categories::list_by_hits(0, COMPACT_LIST_SIZE, 'compact')) {
			$title = i18n::s('Popular');
			$text .= Skin::build_box($title, Skin::build_list($items, 'compact'), 'extra');
		}
	}

	// save for later use
	Cache::put($cache_id, $text, 'articles');
}
$context['extra'] .= $text;

// referrals, if any
if(Surfer::is_associate() || (isset($context['with_referrals']) && ($context['with_referrals'] == 'Y'))) {

	$cache_id = 'categories/index.php#referrals#';
	if(!$text =& Cache::get($cache_id)) {

		// box content
		include_once '../agents/referrals.php';
		$text = Referrals::list_by_hits_for_url($context['url_to_root_parameter'].'categories/index.php');

		// in a sidebar box
		if($text)
			$text = Skin::build_box(i18n::s('Referrals'), $text, 'navigation', 'referrals');

		// save in cache for one hour 60 * 60 = 3600
		Cache::put($cache_id, $text, 'referrals', 3600);

	}

	// in the extra panel
	$context['extra'] .= $text;
}

// render the skin
render_skin($root_cache_id);

?>