<?php
/**
 * the index page for files
 *
 * @todo add a bulk.php to upload files as a table (Justin)
 *
 * For a comprehensive description of files, you should check the database abstraction script
 * at [script]files/files.php[/script].
 *
 * This page list recent files to any surfer.
 * Following restrictions apply:
 * - anonymous users can see only active files (the 'active' field == 'Y')
 * - members can see active and restricted files ('active field == 'Y' or 'R')
 * - associates can see all files
 *
 * Of course files are filtered in the displayed list depending on restriction level and surfer ability.
 * Moreover, non-associates will only see files attached to public published non-expired articles.
 *
 * The main menu has navigation links to browse files by page, for sites that have numerous files.
 *
 * Files are displayed using the decorated layout.
 *
 * The extra panel has following components:
 * - A link to the rss feed for public files, as an extra box
 * - The list of most popular files, if any, as an extra box
 * - Top popular referrals, if any
 *
 * Accept following invocations:
 * - index.php (view the 20 top files)
 * - index.php/2 (view files 41 to 60)
 * - index.php?page=2 (view files 41 to 60)
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// common definitions and initial processing
include_once '../shared/global.php';
include_once '../feeds/feeds.php'; // podcast feed
include_once 'files.php';

// which page should be displayed
$page = 1;
if(isset($_REQUEST['page']))
	$page = $_REQUEST['page'];
elseif(isset($context['arguments'][0]))
	$page = $context['arguments'][0];
$page = strip_tags($page);

// root cache id for this page
$root_cache_id = 'files/index.php/'.$page;

// render a previous version, if any
Cache::pass_through($root_cache_id);

// load localized strings
i18n::bind('files');

// load the skin
load_skin('files');

// page size
$items_per_page = 50;

// the title of the page
$context['page_title'] = i18n::s('All files');

// count files in the database
$stats = Files::stat();
if($stats['count'])
	$context['page_menu'] = array_merge($context['page_menu'], array('_count' => sprintf(i18n::ns('1&nbsp;file', '%d&nbsp;files', $stats['count']), $stats['count'])));

// navigation commands for files, if necessary
if($stats['count'] > $items_per_page) {
	$home = 'files/index.php';
	if(isset($context['with_friendly_urls']) && ($context['with_friendly_urls'] == 'Y'))
		$prefix = $home.'/';
	elseif(isset($context['with_friendly_urls']) && ($context['with_friendly_urls'] == 'R'))
		$prefix = $home.'/';
	else
		$prefix = $home.'?page=';
	$context['page_menu'] = array_merge($context['page_menu'], Skin::navigate($home, $prefix, $stats['count'], $items_per_page, $page));
}

// command to review files
if(Surfer::is_member())
	$context['page_menu'] = array_merge($context['page_menu'], array( 'files/review.php' => i18n::s('Review files') ));

// command to check the database
if(Surfer::is_associate())
	$context['page_menu'] = array_merge($context['page_menu'], array( 'files/check.php' => i18n::s('Maintenance') ));

// configuration panel
if(Surfer::is_associate())
	$context['page_menu'] = array_merge($context['page_menu'], array( 'files/configure.php' => i18n::s('Configure files') ));

// the list of files
$cache_id = 'files/index.php#files_by_date#'.$page;
if(!$text =& Cache::get($cache_id)) {

	// query the database and layout that stuff
	$offset = ($page - 1) * $items_per_page;
	if(!$text = Files::list_by_date($offset, $items_per_page, 'full')) {
		$text = '<p>'.i18n::s('No file has been uploaded yet.').'</p>';

	// we have an array to format
	} elseif(@count($text))
		$text = Skin::build_list($text, 'decorated');

	// cache this to speed subsequent queries
	Cache::put($cache_id, $text, 'files');
}
$context['text'] .= $text;

// get news from rss
$title = i18n::s('Podcast');
$label = sprintf(i18n::s('You can list new public files through RSS by going %s'), Skin::build_link(Feeds::get_url('files'), 'here', 'xml'));
$context['extra'] .= Skin::build_box($title, $label, 'extra');

// side bar with the list of most popular files
$cache_id = 'files/index.php#files_by_hits';
if(!$text =& Cache::get($cache_id)) {
	if($items = Files::list_by_hits(0, COMPACT_LIST_SIZE, 'compact')) {
		$text =& Skin::build_box(i18n::s('Popular'), Skin::build_list($items, 'compact'), 'extra');
	}
	Cache::put($cache_id, $text, 'files');
}
$context['extra'] .= $text;

// side boxes for related categories, if any
$cache_id = 'files/index.php#displayed_categories';
if(!$text =& Cache::get($cache_id)) {

	// one box per category
	include_once '../categories/categories.php';
	if($categories = Categories::list_by_date_for_display('file:index', 0, 7, 'raw')) {
		foreach($categories as $id => $attributes) {

			// link to the category page from the box title
			$label =& Skin::build_box_title(Skin::strip($attributes['title']), Categories::get_url($id), i18n::s('Visit the category'));

			// box content
			if($items = Members::list_articles_by_date_for_anchor('category:'.$id, 0, COMPACT_LIST_SIZE, 'compact'))
				$text .= Skin::build_box($label, Skin::build_list($items, 'compact'), 'navigation')."\n";
		}
	}
	Cache::put($cache_id, $text, 'categories');
}
$context['extra'] .= $text;

// referrals, if any
if(Surfer::is_associate() || (isset($context['with_referrals']) && ($context['with_referrals'] == 'Y'))) {

	$cache_id = 'files/index.php#referrals#';
	if(!$text =& Cache::get($cache_id)) {

		// box content in a sidebar box
		include_once '../agents/referrals.php';
		if($text = Referrals::list_by_hits_for_url($context['url_to_root_parameter'].'files/index.php'))
			$text =& Skin::build_box(i18n::s('Referrals'), $text, 'navigation', 'referrals');

		// save in cache for one hour 60 * 60 = 3600
		Cache::put($cache_id, $text, 'referrals', 3600);

	}

	// in the extra panel
	$context['extra'] .= $text;
}

// render the skin
render_skin($root_cache_id);

?>