<?php
/**
 * heartbit for idle sessions
 *
 * This script, if invoked regularly, will slightly change session data.
 * This means that the modification time of the file used to store session data will
 * evolve over time, and, therefore, that this file won't be eligible for garbage collection.
 *
 * If the user browser is shutdown no heartbit will take place and session data will be deleted as expected.
 *
 * @see shared/global.php
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @tester Innovatique
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// update session data to avoid garbage collection
if(isset($_SERVER['REMOTE_ADDR']) && !headers_sent() && (session_id() == ''))
	session_start();

// change session data to change related file stamps
if(!isset($_SESSION['heartbit']))
	$_SESSION['heartbit'] = 0;
$_SESSION['heartbit']++;

// refresh the watchdog
$_SESSION['watchdog'] = time();

// do not cache output
if(!headers_sent()) {
	include_once 'shared/safe.php';
	Safe::header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
	Safe::header("Cache-Control: no-store, no-cache, must-revalidate");
	Safe::header("Cache-Control: post-check=0, pre-check=0", false);
	Safe::header("Pragma: no-cache");
}

// time of day, except on HEAD
if(isset($_SERVER['REQUEST_METHOD']) && ($_SERVER['REQUEST_METHOD'] != 'HEAD'))
	echo gmdate('M d Y H:i:s').' GMT';

?>