<?php
/**
 * web service encoder and decoder
 *
 * To implement a client of some web service, you will have:
 * - to implement at least export_request() and import_response()
 * - to extend the list of accepted protocols in services/codec.php
 *
 * @see services/xml_rpc.php
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

Class Codec {

	/**
	 * build a XML request on client side
	 *
	 * [php]
	 * $result = $codec->export_request($service, $parameters);
	 * if(!$result[0])
	 * 	echo $result[1]; // print error code
	 * else
	 * 	... // send xml data from $result[1] to the remote web server
	 * [/php]
	 *
	 * @param string name of the remote service
	 * @param mixed transmitted parameters, if any
	 * @return an array of which the first value indicates call success or failure
	 */
	function export_request($service, $parameters = NULL) {
		return array(FALSE, 'Please overlay export_request()');
	}

	/**
	 * build a XML response on server side
	 *
	 * [php]
	 * $result = $codec->export_response($values, $service);
	 * if(!$result[0])
	 * 	echo $result[1]; // print error code
	 * else
	 * 	... // send xml data from $result[1] to the remote web server
	 * [/php]
	 *
	 * @param mixed transmitted values, if any
	 * @param string name of the remote service, if any
	 * @return an array of which the first value indicates call success or failure
	 */
	function export_response($values=NULL, $service=NULL) {
		return array(FALSE, 'Please overlay export_response()');
	}

	/**
	 * parse a XML request on server side
	 *
	 * @param array the received $HTTP_RAW_POST_DATA
	 * @return array the service called and the related input parameters
	 */
	function import_request($data) {
		return array(FALSE, 'Please overlay import_request()');
	}

	/**
	 * parse a XML response on client side
	 *
	 * @param string the received HTTP body
	 * @param string the received HTTP headers
	 * @param mixed the submitted parameters
	 * @return an array of which the first value indicates call success or failure
	 */
	function import_response($data, $headers=NULL, $parameters=NULL) {
		return array(FALSE, 'Please overlay import_response()');
	}

	/**
	 * load one XML codec handler
	 *
	 * @param string the variant to handle
	 * @return a Codec instance, or NULL if the variant is unknown
	 */
	function initialize($variant) {
		global $context;

		switch($variant) {
		case 'packetwiseAPI':
			include_once $context['path_to_root'].'services/packetwiseapi_codec.php';
			return new packetwiseAPI_Codec();
		case 'slashdot':
			include_once $context['path_to_root'].'services/slashdot_codec.php';
			return new slashdot_Codec();
		case 'XML-RPC':
			include_once $context['path_to_root'].'services/xml_rpc_codec.php';
			return new XML_RPC_Codec();
		}
		return NULL;
	}
}

?>