<?php
/**
 * delete a user profile
 *
 * This script calls for confirmation, then it actually deletes the user.
 *
 * If the file [code]demo.flag[/code] exists, the script assumes that this instance
 * of YACS runs in demonstration mode, and user profiles are not actually deleted.
 *
 * Restrictions apply on this page:
 * - associates are allowed to move forward
 * - surfer is allowed to delete its own profile, except if this is forbidden
 * - permission denied is the default
 *
 * To prevent self-deletion, you have to change the parameter 'users_without_self_deletion'
 * from the configuration panel for users.
 *
 * Accept following invocations:
 * - delete.php/12
 * - delete.php?id=12
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @tester Pierre Robert
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// common definitions and initial processing
include_once '../shared/global.php';

// look for the id
$id = NULL;
if(isset($_REQUEST['id']))
	$id = $_REQUEST['id'];
elseif(isset($context['arguments'][0]))
	$id = $context['arguments'][0];
elseif(Surfer::is_logged())
	$id = Surfer::get_id();
$id = strip_tags($id);

// get the item from the database
$item =& Users::get($id);

// associates can do what they want
if(Surfer::is_associate())
	$permitted = TRUE;

// self-deletion is forbidden
elseif(isset($context['users_without_self_deletion']) && ($context['users_without_self_deletion'] == 'Y'))
	$permitted = FALSE;

// the surfer is allowed to delete its own profile
elseif(isset($item['create_id']) && Surfer::is_creator($item['create_id']))
	$permitted = TRUE;

// the default is to disallow access
else
	$permitted = FALSE;

// load localized strings
i18n::bind('users');

// load the skin
load_skin('users');

// the path to this page
$context['path_bar'] = array( 'users/' => i18n::s('Users') );

// the title of the page
$context['page_title'] = i18n::s('Delete a user');

// not found
if(!isset($item['id'])) {
	Safe::header('Status: 404 Not Found', TRUE, 404);
	Skin::error(i18n::s('No user has the provided id.'));

// permission denied
} elseif(!$permitted) {

	// anonymous users are invited to log in
	if(!Surfer::is_logged())
		Safe::redirect($context['url_to_home'].$context['url_to_root'].'users/login.php?url='.urlencode(Users::get_url($item['id'], 'delete')));

	// permission denied to authenticated user
	Safe::header('Status: 403 Forbidden', TRUE, 403);
	Skin::error(i18n::s('You are not allowed to perform this operation.'));

// no deletion in demo mode
} elseif(isset($_REQUEST['confirm']) && ($_REQUEST['confirm'] == 'yes') && file_exists($context['path_to_root'].'parameters/demo.flag')) {

	// remind the surfer
	Safe::header('Status: 403 Forbidden', TRUE, 403);
	$context['text'] .= '<p>'.i18n::s('This instance of YACS runs in demonstration mode. For security reasons user profiles cannot be actually deleted in this mode.')."</p>\n";

// deletion is confirmed
} elseif(isset($_REQUEST['confirm']) && ($_REQUEST['confirm'] == 'yes')) {

	// close the session on self-deletion
	if(Surfer::get_id() == $id)
		Surfer::reset();

	// delete and go back to the index page
	if(Users::delete($id))
		Safe::redirect($context['url_to_home'].$context['url_to_root'].'users/');

// deletion has to be confirmed
} elseif(isset($_SERVER['REQUEST_METHOD']) && ($_SERVER['REQUEST_METHOD'] == 'POST'))
	Skin::error(i18n::s('The deletion has not been confirmed.'));

// ask for confirmation
else {

	// the submit button
	if(Surfer::is_creator($item['id']))
		$label = i18n::s('Yes, I want to suppress my own profile from this server and log out.');
	else
		$label = i18n::s('Yes, I want to suppress this user');
	$context['text'] .= '<form method="post" action="'.$context['script_url'].'" id="main_form">'."\n"
		.'<p>'.Skin::build_submit_button($label, NULL, NULL, 'confirmed').'</p>'."\n"
		.'<input type="hidden" name="id" value="'.$item['id'].'" />'."\n"
		.'<input type="hidden" name="confirm" value="yes" />'."\n"
		.'</form>'."\n";

    // set the focus
    $context['text'] .= '<script type="text/javascript">'."\n"
    	.'<!--'."\n"
		.'// set the focus on first form field'."\n"
		.'document.getElementById(\'confirmed\').focus();'."\n"
		.'// -->'."\n"
		.'</script>'."\n";

	// user nick name
	if($item['nick_name'])
	    $context['text'] .= Skin::build_block($item['nick_name'], 'title');

	// user full name
	if($item['full_name'])
	    $context['text'] .= '<p>'.$item['full_name']."</p>\n";

	// introduction text, if any
	if($item['introduction'])
		$context['text'] .= Skin::build_block($item['introduction'], 'introduction');

	// beautified description, which is the actual page body
	if($item['description'])
			$context['text'] .= Codes::beautify($item['description'])."\n";

	// count items related to this user
	$context['text'] .= Anchors::stat_related_to('user:'.$item['id'], i18n::s('Following items are attached to this user and will be suppressed as well.'));

}

// render the skin
render_skin();

?>