<?php
/**
 * assign users to any object
 *
 * This script displays assigned users to an anchor, and list users that could be assigned as well.
 *
 * This is the main tool used by associates to assign editors to pages they are managing.
 *
 * Only associates can use this script. This means that editors cannot delegate their power to someone else.
 *
 * Accept following invocations:
 * - select.php?member=article:12
 *
 * If this anchor, or one of its anchor, specifies a specific skin (option keyword '[code]skin_xyz[/code]'),
 * or a specific variant (option keyword '[code]variant_xyz[/code]'), they are used instead default values.
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// common definitions and initial processing
include_once '../shared/global.php';

// users are assigned to
$anchor = NULL;

// some data have been posted by an authenticated member
if(isset($_SERVER['REQUEST_METHOD']) && ($_SERVER['REQUEST_METHOD'] == 'POST') && isset($_REQUEST['member']) && Surfer::is_associate()) {

	// actual update
	if($error = Members::toggle($_REQUEST['anchor'], $_REQUEST['member']))
		Skin::error($error);

	// bind the anchor
	else
		$anchor = get_anchor($_REQUEST['member']);

	// clear everything related to users
	Cache::clear('users');

// find the target anchor in path args (e.g., http:.../users/select.php?member=article:15)
} elseif(isset($_REQUEST['member'])) {

	// get the member object, which is supposed to be a container
	$anchor = get_anchor($_REQUEST['member']);

}

// load localized strings
i18n::bind('users');

// load the skin, maybe with a variant
load_skin('users', $anchor);

// the path to this page
if(is_object($anchor) && $anchor->is_viewable())
    $context['path_bar'] = $anchor->get_path_bar();
else
    $context['path_bar'] = array( 'users/' => i18n::s('Users') );

// the title of the page
if(is_object($anchor))
    $context['page_title'] = sprintf(i18n::s('Users assigned to %s'), $anchor->get_title());
else
	$context['page_title'] = i18n::s('Assign users');

// an anchor is mandatory
if(!is_object($anchor))
	Skin::error(i18n::s('Please provide a valid anchor.'));

// surfer has to be an associate
elseif(!Surfer::is_associate())
	Skin::error(i18n::s('You are not allowed to perform this operation.'));

// build a form to associates some users to this item
else {

	// back to the anchor page
    $context['page_menu'] = array( $anchor->get_url() => sprintf(i18n::s('Back to %s'), $anchor->get_title()) );

	// insert anchor prefix
	if(is_object($anchor))
		$context['text'] .= $anchor->get_prefix();

	// splash
	$context['text'] .= '<p>'.i18n::s('Select or deselect users assigned to this page.').'</p>';

	// the current list of linked users
	$assigned_users = array();
	if(($users = Members::list_users_for_member($anchor->get_reference(), 0, USERS_LIST_SIZE, 'raw')) && count($users)) {

		// browse the list
		foreach($users as $id => $user) {

			// this one has already been assigned
			$assigned_users[] = $id;

			// make an url
			$url = Users::get_url($id, 'view', isset($user['nick_name'])?$user['nick_name']:'');

			// gather information on this user
			$prefix = $suffix = $type = $icon = '';
			if(isset($user['full_name']) && $user['full_name'])
				$label = $user['full_name'].' ('.$user['nick_name'].')';
			else
				$label = $user['nick_name'];

			// build a unlink button for this user
			if(Surfer::is_associate()) {
				$suffix .= BR.'<form method="post" action="'.$context['script_url'].'">'
					.'<input type="hidden" name="anchor" value="user:'.$id.'">'
					.'<input type="hidden" name="member" value="'.encode_field($anchor->get_reference()).'">'
					.Skin::build_submit_button(i18n::s('Unassign'))
					.'</form>';
			}

			// format the item
			$new_users[$url] = array($prefix, $label, $suffix, $type, $icon);

		}

		// display attached users with unlink buttons
		$context['text'] .= Skin::build_list($new_users, 'decorated');

	}

	// the form to link additional users
	if(!is_array($users) || (count($users) < USERS_LIST_SIZE)) {
		$context['text'] .= '<p><form method="post" action="'.$context['script_url'].'">'
			.i18n::s('Assign').' <select name="anchor">'.Users::get_options($assigned_users).'</select>'
			.' '.Skin::build_submit_button(' >> ')
			.'<input type="hidden" name="member" value="'.encode_field($anchor->get_reference()).'">'
			.'</form></p>'."\n";
	}

	// insert anchor suffix
	if(is_object($anchor))
		$context['text'] .= $anchor->get_suffix();

}

// render the skin
render_skin();

?>