<?php
/**
 * view one user profile
 *
 * @todo list assigned files
 * @todo list assigned articles (along assigned sections) (Agnes)
 * @todo add 10 preferred blogs/links
 * @todo add birthday
 *
 * This script displays one user profile. Depending on who the surfer is, more or less information is provided.
 * - surfer is the user: view any detail, may edit the page
 * - surfer is associate: view any detail, may edit and delete the page
 * - else: view the user profile
 *
 * The script also lists sections where this user may act as a managing editor, if any.
 *
 * [deleted]If the user has a Yahoo! address or ICQ number, this script generates HTML codes required to display messenger status.[/deleted]
 *
 * Addresses are showed to the surfer only if the surfer has been duly authenticated as a member, or if
 * this has been explicitly required (parameter [code]users_with_email_display[/code] set in [script]control/configure.php[/script]).
 *
 * @link http://www.hypothetic.org/docs/msn/index.php MSN Messenger Protocol
 * @link http://www.movabletype.org/docs/mttrackback.html TrackBack Technical Specification
 *
 * The list of most recent pages from this user is displayed in the main panel.
 * Only the author and associates can see articles that have not been published yet.
 *
 * Articles appear in the Watch list in following cases:
 * - the user is the initial poster of the page
 * - the user has edited the page
 * - the user has added a comment, or an image, or a file
 * - the surfer has explicitly asked for it
 *
 * The extra panel has following components:
 * - An extra box with shortcuts to contribute to the server, including bookmarklets, if this is the surfer profile
 * - A link to the related rss feed, as an extra box
 * - The list of most popular articles, if any, as an extra box
 * - The list of most popular files, if any, as an extra box
 * - The nearest locations, if any, into an extra box
 * - Means to reference this page, into a sidebar box
 * - Top popular referrals, if any
 * - Browser time shift information, if this is the surfer profile
 *
 * Several HTTP headers, or &lt;meta&gt; attributes of the displayed page, are set dynamically here
 * to help advanced web usage. This includes:
 * - a link to a RSS feed for this user profile (e.g., '&lt;link rel="alternate" href="http://127.0.0.1/yacs/users/feed.php/4038" title="RSS" type="application/rss+xml" /&gt;')
 * - a link to a RDF description of this page (e.g., '&lt;link rel="meta" href="http://127.0.0.1/yacs/users/describe.php/4310" title="rdf" type="application/rdf+xml" /&gt;')
 *
 * Restrictions apply on this page:
 * - associates are allowed to move forward
 * - this is the personal record of the authenticated surfer
 * - access is restricted ('active' field == 'R'), but the surfer is an authenticated member
 * - public access is allowed ('active' field == 'Y')
 * - permission denied is the default
 *
 * Accept following invocations:
 * - view.php (show my profile if I am logged)
 * - view.php/12 (view the first page of the user profile)
 * - view.php?id=12 (view the first page of the user profile)
 * - view.php/12/actions/2 (view the page 2 of the list of actions given to this user)
 * - view.php?id=12&actions=2 (view the page 2 of the list of actions given to this user)
 * - view.php/12/articles/2 (view the page 2 of the list of articles contributed by this user)
 * - view.php?id=12&articles=2 (view the page 2 of the list of articles contributed by this user)
 * - view.php/12/files/2 (view the page 2 of the list of files sent by this user)
 * - view.php?id=12&files=2 (view the page 2 of the list of files sent by this user)
 * - view.php/12/links/1 (view the page 1 of the list of links sent by this user)
 * - view.php?id=12&links=1 (view the page 1 of the list of links sent by this user)
 * - view.php/12/bookmarks/2 (view the page 2 of the list of pages bookmarked by this user)
 * - view.php?id=12&bookmarks=2 (view the page 2 of the list of pages bookmarked by this user)
 *
 * @author Bernard Paques [email]bernard.paques@bigfoot.com[/email]
 * @author GnapZ
 * @tester Moi-meme
 * @tester Guillaume Perez
 * @tester AnsteyER
 * @reference
 * @license http://www.gnu.org/copyleft/lesser.txt GNU Lesser General Public License
 */

// common definitions and initial processing
include_once '../shared/global.php';
include_once '../actions/actions.php';
include_once '../articles/articles.php';
include_once '../files/files.php';
include_once '../links/links.php';
include_once '../locations/locations.php';

// look for the id
$id = NULL;
if(isset($_REQUEST['id']))
	$id = $_REQUEST['id'];
elseif(isset($context['arguments'][0]))
	$id = $context['arguments'][0];
elseif(Surfer::is_logged())
	$id = Surfer::get_id();
$id = strip_tags($id);

// encode ISO-8859-1 argument, if any
if(isset($_SERVER['HTTP_ACCEPT_CHARSET']) && preg_match('/^iso-8859-1/i', $_SERVER['HTTP_ACCEPT_CHARSET']))
	$id = utf8_encode($id);

// view.php?id=12&actions=2
if(!isset($zoom_index) && isset($_REQUEST['actions']) && ($zoom_index = $_REQUEST['actions']))
	$zoom_type = 'actions';

// view.php?id=12&articles=2
if(!isset($zoom_index) && isset($_REQUEST['articles']) && ($zoom_index = $_REQUEST['articles']))
	$zoom_type = 'articles';

// view.php?id=12&files=2
if(!isset($zoom_index) && isset($_REQUEST['files']) && ($zoom_index = $_REQUEST['files']))
	$zoom_type = 'files';

// view.php?id=12&links=2
if(!isset($zoom_index) && isset($_REQUEST['links']) && ($zoom_index = $_REQUEST['links']))
	$zoom_type = 'links';

// view.php?id=12&bookmarks=2
if(!isset($zoom_index) && isset($_REQUEST['bookmarks']) && ($zoom_index = $_REQUEST['bookmarks']))
	$zoom_type = 'bookmarks';

// view.php/12/files/2
if(!isset($zoom_index) && isset($context['arguments'][1]) && isset($context['arguments'][2])) {
	$zoom_type = $context['arguments'][1];
	$zoom_index = $context['arguments'][2];
}

// get the item from the database
$item =& Users::get($id);

// get the related overlay, if any
$overlay = NULL;
include_once '../overlays/overlay.php';
if(isset($item['overlay']))
	$overlay = Overlay::load($item);

// actual capability of current surfer
if(isset($item['id']) && Surfer::get_id() && ($item['id'] == Surfer::get_id()) && ($item['capability'] != '?'))
	Surfer::empower();

// associates can do what they want
if(Surfer::is_associate())
	$permitted = TRUE;

// the record of the authenticated surfer
elseif(isset($item['id']) && Surfer::is_creator($item['id']))
	$permitted = TRUE;

// access is restricted to authenticated member
elseif(isset($item['active']) && ($item['active'] == 'R') && Surfer::is_member())
	$permitted = TRUE;

// public access is allowed
elseif(isset($item['active']) && ($item['active'] == 'Y'))
	$permitted = TRUE;

// the default is to disallow access
else
	$permitted = FALSE;

// load localized strings
i18n::bind('users');

// load the skin
load_skin('users');

// the path to this page
$context['path_bar'] = array( 'users/' => i18n::s('Users') );

// the title of the page
if(isset($item['nick_name']))
	$context['page_title'] = $item['nick_name'];
elseif(isset($item['full_name']))
	$context['page_title'] = $item['full_name'];
else
	$context['page_title'] = i18n::s('Unknown user');

// logged users may download the vcard on first page
if(isset($item['id']) && !isset($zoom_type) && Surfer::is_logged())
	$context['page_menu'] = array_merge($context['page_menu'], array( Users::get_url($item['id'], 'fetch_vcard', isset($item['nick_name'])?$item['nick_name']:'') => i18n::s('vCard') ));

// anyone can modify his own profile; associates can do what they want
if(isset($item['id']) && !isset($zoom_type) && (Surfer::is_creator($item['id']) || Surfer::is_associate())) {
	$context['page_menu'] = array_merge($context['page_menu'], array( Users::get_url($item['id'], 'edit') => i18n::s('Edit') ));
	$context['page_menu'] = array_merge($context['page_menu'], array( Users::get_url($item['id'], 'select_avatar') => i18n::s('Avatar') ));
	if(Surfer::may_upload())
		$context['page_menu'] = array_merge($context['page_menu'], array( 'images/edit.php?anchor=user:'.$item['id'] => i18n::s('Post an image') ));
	$context['page_menu'] = array_merge($context['page_menu'], array( Users::get_url($item['id'], 'password') => i18n::s('Password') ));
}

// only associates can delete user profiles; self-deletion may also be allowed
if(isset($item['id']) && !isset($zoom_type)
	&& (Surfer::is_associate()
		|| (Surfer::is_creator($item['id']) && (!isset($context['users_without_self_deletion']) || ($context['users_without_self_deletion'] != 'Y'))))) {

	$context['page_menu'] = array_merge($context['page_menu'], array( Users::get_url($item['id'], 'delete') => i18n::s('Delete') ));
}

// any member can send a message, if mail has been activated on this server
if(isset($item['id']) && !isset($zoom_type) && Surfer::is_member() && isset($item['email']) && ($item['without_messages'] != 'Y')
	&& isset($context['with_email']) && ($context['with_email'] == 'Y')) {
	$context['page_menu'] = array_merge($context['page_menu'], array( Users::get_url($item['id'], 'mail') => i18n::s('Send a message') ));
}

// any member can submit a new action to another profile
if(isset($item['id']) && !isset($zoom_type) && Actions::are_allowed(NULL, $item))
	$context['page_menu'] = array_merge($context['page_menu'], array( 'actions/edit.php?anchor=user:'.$item['id'] => i18n::s('Post an action') ));

// the print command is provided to logged users, or to any user in Wiki mode
if(isset($item['id']) && !isset($zoom_type) && $permitted && (Surfer::is_logged() || (isset($context['users_with_auto_publish']) && ($context['users_with_auto_publish'] == 'Y')) )) {
	$context['page_menu'] = array_merge($context['page_menu'], array( Users::get_url($item['id'], 'print') => i18n::s('Print') ));
}

// not found -- help web crawlers
if(!isset($item['id'])) {
	Safe::header('Status: 404 Not Found', TRUE, 404);
	Skin::error(i18n::s('No user has the provided id.'));

// permission denied
} elseif(!$permitted) {

	// anonymous users are invited to log in or to register
	if(!Surfer::is_logged())
		Safe::redirect($context['url_to_home'].$context['url_to_root'].'users/login.php?url='.urlencode(Users::get_url($item['id'])));

	// permission denied to authenticated user
	Safe::header('Status: 403 Forbidden', TRUE, 403);
	Skin::error(i18n::s('You are not allowed to perform this operation.'));

// display the user profile
} else {

	// initialize the rendering engine
	Codes::initialize(Users::get_url($item['id'], 'view', $item['nick_name']));

	// the user icon, if any
	if(isset($item['avatar_url']))
		$context['page_image'] = $item['avatar_url'];

	// cache the main content
	$cache_id = 'users/view.php?id='.$item['id'].'#content';
	if(!$text =& Cache::get($cache_id)) {

		// details
		$details = array();

		// the number of posts
		if(isset($item['posts']) && ($item['posts'] > 1))
			$details[] = sprintf(i18n::s('%d posts'), $item['posts']);

		// the date of last login
		if(Surfer::is_associate() && isset($item['login_date']) && $item['login_date'])
			$details[] = sprintf(i18n::s('last login %s'), Skin::build_date($item['login_date']));

		// the date of registration
		if(isset($item['create_date']) && $item['create_date'])
			$details[] = sprintf(i18n::s('registered %s'), Skin::build_date($item['create_date']));

		// combine these three items into one
		if(count($details))
			$details = array( implode(', ', $details) );

		// the capability field is displayed only to logged users
		if(!Surfer::is_logged())
			;
		elseif($item['capability'] == 'A') {

			// add links to contribute to this site
			if(Surfer::is_creator($id))
				$details[] = sprintf(i18n::s('As an associate of this community, you may contribute freely to any part of this server. Enjoy! %s, %s, %s'), Skin::build_link('articles/edit.php', i18n::s('Write a page')), Skin::build_link('images/edit.php', i18n::s('Post a photo')), Skin::build_link('files/edit.php', i18n::s('Share a file')))."\n";
			else
				$details[] = i18n::s('Associate of this community, that has specific rights (and duties) on this server.');

		} elseif($item['capability'] == 'M') {

			// add links to contribute to this site
			if(Surfer::is_creator($id))
				$details[] = sprintf(i18n::s('As a member of this community, you may access freely to most part of this server. Enjoy! %s, %s, %s'), Skin::build_link('articles/edit.php', i18n::s('Write a page')), Skin::build_link('images/edit.php', i18n::s('Post a photo')), Skin::build_link('files/edit.php', i18n::s('Share a file')))."\n";
			else
				$details[] = i18n::s('Member of this community, with contribution rights to this server.');

		} elseif($item['capability'] == 'S') {
			if(Surfer::is_creator($id))
				$details[] = i18n::s('As a subscriber of this community, you may browse public pages and receive newsletters periodically.');
			else
				$details[] = i18n::s('Subscriber of this community, allowed to browse public pages and to receive e-mail newsletters.');

		} elseif($item['capability'] == '?') {
			if(Surfer::is_associate())
				$details[] = EXPIRED_FLAG.i18n::s('This surfer has been banned and cannot authenticate.');
		}

		// restricted or hidden profile
		if(isset($item['active']) && ($item['active'] != 'Y')) {

			if(($item['active'] == 'R') && Surfer::is_member())
				$details[] = RESTRICTED_FLAG.' '.i18n::s('Visibility is restricted to logged members.');
			elseif(($item['active'] == 'N') && Surfer::is_associate())
				$details[] = PRIVATE_FLAG.' '.i18n::s('Visibility is restricted to associates.');

		}

		// locked profile
		if(Surfer::is_member() && preg_match('/\blocked\b/i', $item['options']) ) {
			if(Surfer::is_associate())
				$details[] = LOCKED_FLAG.' '.i18n::s('Profil is locked, but you can modify it.');
			elseif(Surfer::is_creator($item['id']))
				$details[] = LOCKED_FLAG.' '.i18n::s('You are not allowed to modify your user profile.');
			else
				$details[] = LOCKED_FLAG.' '.i18n::s('Profil is locked, except to associates.');
		}

		// provide details
		if(count($details))
			$text .= '<p class="details">'.implode(BR."\n", $details).'</p>';

		// the full name
		if(isset($item['full_name']) && $item['full_name'])
			$text .= '<p>'.sprintf(i18n::s('Full name: %s'), $item['full_name'])."</p>\n";

		// web address, if any
		if(isset($item['web_address']) && $item['web_address'])
			$text .= '<p>'.sprintf(i18n::s('Web address: %s'), Skin::build_link($item['web_address'], $item['web_address'], 'external'))."</p>\n";

		// following fields are showed only to authenticated members, to not let robots nor subscribers steal addresses
		if((!isset($context['users_with_email_display']) || ($context['users_with_email_display'] != 'N')) && (Surfer::is_member() || ($context['users_with_email_display'] == 'Y'))) {

			// email address - not showed to anonymous surfers for spam protection
			if(isset($item['email']) && $item['email']) {
				if(Surfer::is_creator($id))
					$label = i18n::s('Your e-mail address: %s %s');
				else
					$label = i18n::s('E-mail address: %s %s');

				if(isset($context['with_email']) && ($context['with_email'] == 'Y'))
					$url = Users::get_url($id, 'mail');
				else
					$url = 'mailto:'.$item['email'];

				if(isset($item['with_newsletters']) && ($item['with_newsletters'] == 'Y'))
					$suffix = '';
				else
					$suffix = i18n::s('(do not wish to receive newsletters)');

				$text .= '<p>'.sprintf(i18n::s($label), Skin::build_link($url, $item['email'], 'email'), $suffix)."</p>\n";
			}

			// put contact addresses in a table
			$rows = array();

			// a clickable aim address
			if(isset($item['aim_address']) && $item['aim_address'])
				$rows[] = array(Skin::build_presence($item['aim_address'], 'aim'), sprintf(i18n::s('AIM Screenname: %s'), $item['aim_address']));

			// a clickable icq number
			if(isset($item['icq_address']) && $item['icq_address'])
				$rows[] = array(Skin::build_presence($item['icq_address'], 'icq'), sprintf(i18n::s('ICQ Number: %s'), $item['icq_address']));

			// a clickable irc address
			if(isset($item['irc_address']) && $item['irc_address'])
				$rows[] = array(Skin::build_presence($item['irc_address'], 'irc'), sprintf(i18n::s('IRC address: %s'), $item['irc_address']));

			// a clickable jabber address
			if(isset($item['jabber_address']) && $item['jabber_address'])
				$rows[] = array(Skin::build_presence($item['jabber_address'], 'jabber'), sprintf(i18n::s('Jabber Messenger: %s'), $item['jabber_address']));

			// a clickable msn address
			if(isset($item['msn_address']) && $item['msn_address'])
				$rows[] = array(Skin::build_presence($item['msn_address'], 'msn'), sprintf(i18n::s('MSN Instant Messenger: %s'), $item['msn_address']));

			// a clickable skype address
			if(isset($item['skype_address']) && $item['skype_address'])
				$rows[] = array(Skin::build_presence($item['skype_address'], 'skype'), sprintf(i18n::s('Skype: %s'), $item['skype_address']));

			// a clickable yahoo address -- the on-line status indicator requires to be connected to the Internet
			if(isset($item['yahoo_address']) && $item['yahoo_address'])
 				$rows[] = array(Skin::build_presence($item['yahoo_address'], 'yahoo'), sprintf(i18n::s('Yahoo Messenger: %s'), $item['yahoo_address'])
					.' <img src="http://opi.yahoo.com/online?u='.$item['yahoo_address'].'&amp;m=g&amp;t=1" alt="Yahoo Online Status Indicator"'.EOT);

			if(count($rows))
				$text .= Skin::table(NULL, $rows);

		// motivate people to register
		} elseif(!Surfer::is_logged())
			$text .= '<p>'.i18n::s('Please authenticate to view e-mail and instant messaging contact information, for this user.')."</p>\n";

		// get text related to the overlay, if any
		if(is_object($overlay))
			$text .= $overlay->get_text('view', $item);

		// the full text
		if(isset($item['description']) && $item['description'])
			$text .= '<p>'.sprintf(i18n::s('About this user: %s'), Codes::beautify($item['description']))."<p />\n";

		//
		// preferences, but only to authenticated surfers
		//

		if(Surfer::is_logged()) {

			$box['title'] = i18n::s('Preferences');
			$box['text'] = '';

			// preferred editor
			if(isset($item['editor']) && ($item['editor'] == 'fckeditor'))
				$label = Skin::build_link('http://www.fckeditor.net/', i18n::s('FCKeditor'), 'external');
			elseif(isset($item['editor']) && ($item['editor'] == 'tinymce'))
				$label = Skin::build_link('http://tinymce.moxiecode.com/', i18n::s('TinyMCE'), 'external');
			else
				$label = i18n::s('Textarea &amp; YACS codes');
			$box['text'] .= '<p>'.sprintf(i18n::s('Editor: %s'), $label).'</p>'."\n";

			// use a folded box
			if($box['text'])
				$text .= Skin::build_box($box['title'], $box['text'], 'folder');

		}

		//
		// other data
		//

		$box['title'] = i18n::s('Other data');
		$box['text'] = '';

		// from where
		if(isset($item['from_where']) && $item['from_where'])
			$box['text'] .= '<p>'.sprintf(i18n::s('From: %s'), Codes::beautify($item['from_where'])).'</p>'."\n";

		// tag line
		if(isset($item['introduction']) && $item['introduction'])
			$box['text'] .= '<p>'.sprintf(i18n::s('Tag-line: %s'), Codes::beautify($item['introduction'])).'</p>'."\n";

		// interests
		if(isset($item['interests']) && $item['interests'])
			$box['text'] .= '<p>'.sprintf(i18n::s('Interests: %s'), Codes::beautify($item['interests'])).'</p>'."\n";

		// signature
		if(isset($item['signature']) && $item['signature'])
			$box['text'] .= '<p>'.sprintf(i18n::s('Signature: %s'), BR.Codes::beautify($item['signature'])).'</p>'."\n";

		// pgp key
		if(isset($item['pgp_key']) && $item['pgp_key'])
			$box['text'] .= '<p>'.sprintf(i18n::s('PGP key or certificate: %s'), BR.'<span style="font-size: 50%">'.$item['pgp_key'].'</span>').'</p>'."\n";

		// use a folded box
		if($box['text'])
			$text .= Skin::build_box($box['title'], $box['text'], 'folder');

		// save in cache if no dynamic element
		if(!preg_match('/\[table=(.+?)\]/i', $item['description']))
			Cache::put($cache_id, $text, 'user:'.$item['id']);
	}
	$context['text'] .= $text;

	//
	// files attached to this user profile
	//

	// the list of attached files if not at another follow-up page
	if(!isset($zoom_type) || ($zoom_type == 'files')) {

		// cache the section
		if(!isset($zoom_index))
			$zoom_index = 1;
		$cache_id = 'users/view.php?id='.$item['id'].'#files#'.$zoom_index;
		if(!$text =& Cache::get($cache_id)) {

			// build a complete box
			$box['bar'] = array();
			$box['text'] = '';

			// list files by date
			$offset = ($zoom_index - 1) * FILES_PER_PAGE;
			$items = Files::list_by_date_for_anchor('user:'.$item['id'], $offset, FILES_PER_PAGE, 'no_author');

			// the command to post a new file
			if((Surfer::is_associate() || (Surfer::get_id() == $item['id'])) && Files::are_allowed(NULL, $item)) {
				$url = 'files/edit.php?anchor=user:'.$item['id'];
				$box['bar'] = array_merge($box['bar'], array( $url => i18n::s('Attach a file to this page') ));
			}

			// actually render the html for the section
			$box['title'] = i18n::s('Related files');
			if(@count($items))
				$box['title'] .= ' ('.count($items).')';
			if(@count($box['bar']))
				$box['text'] .= Skin::build_list($box['bar'], 'menu_bar')."\n";
			if(@count($items))
				$box['text'] .= Skin::build_list($items, 'decorated');
			if($box['text'])
				$text =& Skin::build_box($box['title'], $box['text'], 'folder', 'files');

			// save in cache
			Cache::put($cache_id, $text, 'files');
		}

		// embed a box for files
		$context['text'] .= $text;
	}

	//
	// actions attached to this user profile
	//

	// the list of attached actions if not at another follow-up page
	if(!isset($zoom_type) || ($zoom_type == 'actions')) {

		// cache the section
		if(!isset($zoom_index))
			$zoom_index = 1;
		$cache_id = 'users/view.php?id='.$item['id'].'#actions#'.$zoom_index;
		if(!$text =& Cache::get($cache_id)) {

			// build a complete box
			$box['bar'] = array();
			$box['text'] = '';

			// list actions by date
			$offset = ($zoom_index - 1) * ACTIONS_PER_PAGE;
			$items = Actions::list_by_date_for_anchor('user:'.$item['id'], $offset, ACTIONS_PER_PAGE);

			// the command to post a new action
			if((Surfer::is_associate() || (Surfer::get_id() == $item['id'])) && Actions::are_allowed(NULL, $item)) {
				$url = 'actions/edit.php?anchor=user:'.$item['id'];
				$box['bar'] = array_merge($box['bar'], array( $url => i18n::s('New action') ));
			}

			// actually render the html for the section
			if(@count($items))
				$box['title'] .= ' ('.count($items).')';
			if(@count($box['bar']))
				$box['text'] .= Skin::build_list($box['bar'], 'menu_bar')."\n";
			if(@count($items))
				$box['text'] .= Skin::build_list($items, 'decorated');
			if($box['text'])
				$text =& Skin::build_box(i18n::s('On-going actions'), $box['text'], 'folder', 'actions');

			// save in cache
			Cache::put($cache_id, $text, 'actions');
		}

		// embed a box for actions
		$context['text'] .= $text;
	}

	//
	// sections managed by this user
	//

	// cache related sections
	$cache_id = 'users/view.php?id='.$item['id'].'#sections';
	if(!$text =& Cache::get($cache_id)) {

		// sections managed or accessed by this user
		$items = Members::list_sections_by_title_for_anchor('user:'.$item['id'], 0, COMPACT_LIST_SIZE, 'compact');

		// set title before appending extra items
		if(($item['capability'] == 'M') || ($item['capability'] == 'A'))
			$box['title'] = i18n::s('Managed sections');
		else
			$box['title'] = i18n::s('Subscribed sections');
		if(@count($items))
			$box['title'] .= ' ('.count($items).')';

		// the maximum number of personal sections per user
		if(!isset($context['users_maximum_managed_sections']))
			$context['users_maximum_managed_sections'] = 0;

		// offer to extend personal spaces
		if(Surfer::is_member() && (Surfer::get_id() == $item['id']) && (count(Surfer::personal_sections()) < $context['users_maximum_managed_sections']))
			$items[ 'sections/new.php' ] = array(NULL, i18n::s('Create a blog, a discussion board, or another personal web space'), NULL, 'shortcut', NULL);

		// only associates can assign editors and readers
		if(Surfer::is_associate()) {

			// members can become editors of some sections
			if(($item['capability'] == 'M') || ($item['capability'] == 'A'))
				$label = i18n::s('Select sections managed by this user');

			// subscribers can become readers of some sections
			else
				$label = i18n::s('Select sections accessed by this user');

			$items[ 'sections/select.php?anchor=user:'.$item['id'] ] = array(NULL, $label, NULL, 'shortcut', NULL);
		}

		// build a nice box
		if(count($items)) {
			$text =& Skin::build_box($box['title'], Skin::build_list($items, 'compact'), 'folder', 'managed_sections');
		}

		// save in cache
		Cache::put($cache_id, $text, 'sections');

	}
	$context['text'] .= $text;

	//
	// the articles
	//

	// the list of contributed articles if not at another follow-up page
	if(!isset($zoom_type) || ($zoom_type == 'articles')) {

		// cache the section
		if(!isset($zoom_index))
			$zoom_index = 1;
		$cache_id = 'users/view.php?id='.$item['id'].'#contributed_articles#'.$zoom_index;
		if(!$text =& Cache::get($cache_id)) {

			// build a complete box
			$box['bar'] = array();
			$box['text'] = '';

			// count the number of articles for this user
			$stats = Articles::stat_for_author($item['id']);
			if($stats['count'] > ARTICLES_PER_PAGE)
				$box['bar'] = array_merge($box['bar'], array('_count' => sprintf(i18n::ns('1&nbsp;article', '%d&nbsp;articles', $stats['count']), $stats['count'])));

			// navigation commands for articles
			$home = Users::get_url($item['id'], 'view', $item['nick_name']);
			$prefix = Users::get_url($item['id'], 'navigate', 'articles');
			$box['bar'] = array_merge($box['bar'],
				Skin::navigate($home, $prefix, $stats['count'], ARTICLES_PER_PAGE, $zoom_index));

			// the command to post a new article
			if((Surfer::get_id() == $item['id']) && Surfer::is_member())
				$box['bar'] = array_merge($box['bar'], array( 'articles/edit.php' => i18n::s('Create a page') ));

			// compute offset from list beginning
			$offset = ($zoom_index - 1) * ARTICLES_PER_PAGE;

			// list articles by date
			$items = Articles::list_by_date_for_author($item['id'], $offset, ARTICLES_PER_PAGE, 'compact');

			// actually render the html for the section
			if(@count($box['bar']))
				$box['text'] .= Skin::build_list($box['bar'], 'menu_bar')."\n";
			if(@count($items))
				$box['text'] .= Skin::build_list($items, 'compact');
			if($box['text'])
				$text =& Skin::build_box(i18n::s('Recent pages'), $box['text'], 'section', 'contributed_articles');

			// save in cache
			Cache::put($cache_id, $text, 'articles');
		}

		// embed a box for articles
		$context['text'] .= $text;
	}

	// the list of contributed files if not at another follow-up page
	if(!isset($zoom_type)) {

		// cache the section
		if(!isset($zoom_index))
			$zoom_index = 1;
		$cache_id = 'users/view.php?id='.$item['id'].'#contributed_files#'.$zoom_index;
		if(!$text =& Cache::get($cache_id)) {

			// build a complete box
			$box['text'] = '';

			// list files by date
			$items = Files::list_by_date_for_author($item['id'], 0, 20, 'compact');

			// actually render the html for the section
			if(@count($items))
				$box['text'] .= Skin::build_list($items, 'compact');
			if($box['text'])
				$text =& Skin::build_box(i18n::s('Recent files'), $box['text'], 'section', 'contributed_files');

			// save in cache
			Cache::put($cache_id, $text, 'files');
		}

		// embed a box for files
		$context['text'] .= $text;
	}

	//
	// the watch list
	//

	// the list of attached bookmarks if not at another follow-up page
	if(!isset($zoom_type) || ($zoom_type == 'bookmarks')) {

		// put it in a section box
		$box['bar'] = array();
		$box['content'] = '';

		// list watched sections first, up to 50
		$items = Members::list_sections_by_date_for_member('user:'.$item['id'], 0, 50, 'compact');
		if(@count($items))
			$box['content'] .= Skin::build_list($items, 'compact');

		// compute offset from list beginning
		$offset = ($zoom_index - 1) * BOOKMARKS_PER_PAGE;

		// list watched articles by date
		$items = Members::list_articles_by_date_for_member('user:'.$item['id'], $offset, BOOKMARKS_PER_PAGE, 'compact');

		// the navigation commands for bookmarks
//		$box['bar'] = array_merge($box['bar'],
//			Skin::navigate('users/view.php?id='.$item['id'], 'users/view.php?id='.$item['id'].$item['id'].'/bookmarks/', $items, BOOKMARKS_PER_PAGE, $zoom_index));

		// actually render the html for the section
		if(@count($box['bar']))
			$box['content'] = Skin::build_list($box['bar'], 'menu_bar')."\n".$box['content'];
		if(@count($items))
			$box['content'] .= Skin::build_list($items, 'compact');

		if($box['content'])
			$context['text'] .= Skin::build_box(i18n::s('Watch list'), $box['content'], 'section', 'watch_list');

	}

	//
	// links attached to this page
	//

	// the list of attached links if not at another follow-up page
	if(!isset($zoom_type) || ($zoom_type == 'links')) {

		// cache the section
		if(!isset($zoom_index))
			$zoom_index = 1;
		$cache_id = 'users/view.php?id='.$item['id'].'#links#'.$zoom_index;
		if(!$text =& Cache::get($cache_id)) {

			// build a complete box
			$box['bar'] = array();
			$box['text'] = '';

			// list links by date
			$offset = ($zoom_index - 1) * LINKS_PER_PAGE;
			if(preg_match('/\blinks_by_title\b/i', $item['options']))
				$items = Links::list_by_title_for_anchor('user:'.$item['id'], $offset, LINKS_PER_PAGE, 'no_author');
			else
				$items = Links::list_by_date_for_anchor('user:'.$item['id'], $offset, LINKS_PER_PAGE, 'no_author');

			// the command to post a new link
			if(((Surfer::get_id() == $item['id']) || Surfer::is_associate()) && Links::are_allowed(NULL, $item)) {
				$url = 'links/edit.php?anchor=user:'.$item['id'];
				$box['bar'] = array_merge($box['bar'], array( $url => i18n::s('Attach a link to this page') ));
			}

			// actually render the html
			if(@count($box['bar']))
				$box['text'] .= Skin::build_list($box['bar'], 'menu_bar')."\n";
			if(@count($items))
				$box['text'] .= Skin::build_list($items, 'decorated');
			if($box['text'])
				$text =& Skin::build_box(i18n::s('Related links'), $box['text'], 'section', 'links');

			// save in cache
			Cache::put($cache_id, $text, 'links');
		}

		// embed a box for links
		$context['text'] .= $text;
	}

	//
	// populate the extra panel
	//

	// on my page, offer tools to create articles
	if(isset($item['id']) && Surfer::is_member() && ($item['id'] == Surfer::get_id())) {

		// contribute now
		$label = i18n::s('Contribute now:')."\n";;

		// shortcuts

		$link_list = array(
				'articles/edit.php'	=> array(NULL, i18n::s('Write a page'), NULL, 'shortcut', NULL, i18n::s('Use a web form to post a new article')),
				'images/edit.php'	=> array(NULL, i18n::s('Post a photo'), NULL, 'shortcut', NULL, i18n::s('Upload a picture from your computer')),
				'files/edit.php'	=> array(NULL, i18n::s('Share a file'), NULL, 'shortcut', NULL, i18n::s('Pick some file, and upload it for immediate podcast')),
				);

		$label .= Skin::build_list($link_list, 'compact');

		// contribute later
		$label .= BR.i18n::s('Bookmark for later use:')."\n".'<ul>'."\n";

		// the blogging bookmarklet uses YACS codes
		$bookmarklet = "javascript:function findFrame(f){var i;try{isThere=f.document.selection.createRange().text;}catch(e){isThere='';}if(isThere==''){for(i=0;i<f.frames.length;i++){findFrame(f.frames[i]);}}else{s=isThere}return s}"
			."var s='';"
			."d=document;"
			."s=d.selection?findFrame(window):window.getSelection();"
			."window.location='".$context['url_to_home'].$context['url_to_root']."articles/edit.php?"
				."title='+escape(d.title)+'"
				."&amp;text='+escape('%22'+s+'%22%5Bnl]-- %5Blink='+d.title+']'+d.location+'%5B/link]')+'"
				."&amp;source='+escape(d.location);";
		$label .= '<li><a href="'.$bookmarklet.'">'.sprintf(i18n::s('Blog at %s'), $context['site_name']).'</a></li>'."\n";

		// the bookmarking bookmarklet
		$bookmarklet = "javascript:function findFrame(f){var i;try{isThere=f.document.selection.createRange().text;}catch(e){isThere='';}if(isThere==''){for(i=0;i<f.frames.length;i++){findFrame(f.frames[i]);}}else{s=isThere}return s}"
			."var s='';"
			."d=document;"
			."s=d.selection?findFrame(window):window.getSelection();"
			."window.location='".$context['url_to_home'].$context['url_to_root']."links/edit.php?"
				."link='+escape(d.location)+'"
				."&amp;title='+escape(d.title)+'"
				."&amp;text='+escape(s);";
		$label .= '<li><a href="'.$bookmarklet.'">'.sprintf(i18n::s('Bookmark at %s'), $context['site_name']).'</a></li>'."\n";

		// end of bookmarklets
		$label .= '</ul>'."\n";

		// an extra box
		$context['extra'] .= Skin::build_box(i18n::s('Tools'), $label, 'extra');
	}

	// get news from rss
	if(isset($item['id']) && isset($item['capability']) && (($item['capability'] == 'A') || ($item['capability'] == 'M')) && (!isset($context['pages_without_feed']) || ($context['pages_without_feed'] != 'Y')) ) {

		$content = Skin::build_link($context['url_to_home'].$context['url_to_root'].Users::get_url($item['id'], 'feed'), i18n::s('recent articles'), 'xml').BR
			.join(BR, Skin::build_subscribers($context['url_to_home'].$context['url_to_root'].Users::get_url($item['id'], 'feed'), $item['nick_name']));

		$context['extra'] .= Skin::build_box(i18n::s('Stay tuned'), $content, 'extra', 'feeds');
	}

	// the most popular articles for this user
	$cache_id = 'users/view.php?id='.$item['id'].'#popular_articles';
	if(!$text =& Cache::get($cache_id)) {
		$items = Articles::list_by_hits_for_author($item['id'], 0, COMPACT_LIST_SIZE, 'hits');
		if(@count($items))
			$text .= Skin::build_box(i18n::s('Top articles'), Skin::build_list($items, 'compact'), 'extra', 'top_articles');

		// save in cache
		Cache::put($cache_id, $text, 'articles');
	}
	$context['extra'] .= $text;

	// the most popular files for this user
	$cache_id = 'users/view.php?id='.$item['id'].'#popular_files';
	if(!$text =& Cache::get($cache_id)) {
		$items = Files::list_by_hits_for_author($item['id'], 0, COMPACT_LIST_SIZE);
		if(@count($items))
			$text .= Skin::build_box(i18n::s('Top files'), Skin::build_list($items, 'compact'), 'extra', 'top_files');

		// save in cache
		Cache::put($cache_id, $text, 'files');
	}
	$context['extra'] .= $text;

	// neighbours, if any
	if(!isset($zoom_type)) {

		// cache the section
		$cache_id = 'users/view.php?id='.$item['id'].'#neighbours#';
		if(!$text =& Cache::get($cache_id)) {

			// locate up to 5 neighbours
			$items = Locations::list_by_distance_for_anchor('user:'.$item['id'], 1, 5);
			if(@count($items))
				$text .= Skin::build_box(i18n::s('Neighbours'), Skin::build_list($items, 'compact'), 'extra', 'locations');

			// avoid subsequent queries
			else
				$text = ' ';

			Cache::put($cache_id, $text, 'locations');

		}
		$context['extra'] .= $text;
	}

	// how to reference this page
	if(Surfer::is_member() && !isset($zoom_type) && (!isset($context['pages_without_reference']) || ($context['pages_without_reference'] != 'Y')) )
		$context['extra'] .= Skin::build_box(i18n::s('Reference this page'), Codes::beautify(sprintf(i18n::s('Here, use code [escape][user=%s][/escape][nl]Elsewhere, bookmark the [link=full link]%s[/link]'), ($item['nick_name']?$item['nick_name']:$item['id']), $context['url_to_root'].Users::get_url($item['id'], 'view', isset($item['nick_name'])?$item['nick_name']:''))), 'navigation', 'reference');

	// referrals, if any
	if(!isset($zoom_type) && (Surfer::is_associate() || (isset($context['with_referrals']) && ($context['with_referrals'] == 'Y')))) {

		// cache the box
		$cache_id = 'users/view.php?id='.$item['id'].'#referrals#';
		if(!$text =& Cache::get($cache_id)) {

			// box content
			include_once '../agents/referrals.php';
			$text = Referrals::list_by_hits_for_url($context['url_to_root_parameter'].Users::get_url($item['id']));

			// in a sidebar box
			if($text)
				$text =& Skin::build_box(i18n::s('Referrals'), $text, 'navigation', 'referrals');

			// save in cache for one hour 60 * 60 = 3600
			Cache::put($cache_id, $text, 'referrals', 3600);

		}

		// in the extra panel
		$context['extra'] .= $text;

	}

	// display workstation time offset
	if(!isset($zoom_type) && Surfer::is_logged() && Surfer::is_creator($item['id'])) {

		// box content
		$text = '<script type="text/javascript">'."\n"
			.'<!--'."\n"
			.'now = new Date();'."\n"
			.'offset = (-now.getTimezoneOffset() / 60);'."\n"
			.'document.writeln("<p>UTC " + ((offset > 0) ? "+" : "-") + offset + " '.i18n::s('hour(s)').'</p>");'."\n"
			.'// -->'."\n"
			.'</script>'."\n";

		// in the extra panel
		$context['extra'] .= Skin::build_box(i18n::s('Browser GMT offset'), $text, 'navigation', 'time_offset');

	}

	//
	// meta information
	//

	// a meta link to a feeding page
	$context['page_header'] .= "\n".'<link rel="alternate" href="'.$context['url_to_root'].Users::get_url($item['id'], 'feed').'" title="RSS" type="application/rss+xml"'.EOT;

	// a meta link to a description page (actually, rdf)
	$context['page_header'] .= "\n".'<link rel="meta" href="'.$context['url_to_root'].Users::get_url($item['id'], 'describe').'" title="Meta Information" type="application/rdf+xml"'.EOT;

	// implement the trackback interface
	$permanent_link = $context['url_to_home'].$context['url_to_root'].Users::get_url($item['id'], 'view', isset($item['nick_name'])?$item['nick_name']:'');
	if(isset($context['with_friendly_urls']) && (isset($context['with_friendly_urls']) && (isset($context['with_friendly_urls']) && ($context['with_friendly_urls'] == 'Y'))))
		$trackback_link = $context['url_to_home'].$context['url_to_root'].'links/trackback.php/user/'.$item['id'];
	else
		$trackback_link = $context['url_to_home'].$context['url_to_root'].'links/trackback.php?anchor=user:'.$item['id'];
	$context['page_header'] .= "\n".'<!--'
		."\n".'<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"'
		."\n".'         xmlns:dc="http://purl.org/dc/elements/1.1/"'
		."\n".'         xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">'
		."\n".'<rdf:Description'
		."\n".'	trackback:ping="'.$trackback_link.'"'
		."\n".'	dc:identifier="'.$permanent_link.'"'
		."\n".'	rdf:about="'.$permanent_link.'" />'
		."\n".'</rdf:RDF>'
		."\n".'-->';

}

// render the skin
render_skin();

?>